Struct IntoIter
struct IntoIter<T> { ... }
Iterator over the bytes contained by the buffer.
Examples
Basic usage:
use Bytes;
let buf = from;
let mut iter = buf.into_iter;
assert_eq!;
assert_eq!;
assert_eq!;
assert_eq!;
Implementations
impl<T> IntoIter<T>
fn new(inner: T) -> IntoIter<T>Creates an iterator over the bytes contained by the buffer.
Examples
use Bytes; let buf = from_static; let mut iter = buf.into_iter; assert_eq!; assert_eq!; assert_eq!; assert_eq!;fn into_inner(self: Self) -> TConsumes this
IntoIter, returning the underlying value.Examples
use ; let buf = from; let mut iter = buf.into_iter; assert_eq!; let buf = iter.into_inner; assert_eq!;fn get_ref(self: &Self) -> &TGets a reference to the underlying
Buf.It is inadvisable to directly read from the underlying
Buf.Examples
use ; let buf = from; let mut iter = buf.into_iter; assert_eq!; assert_eq!;fn get_mut(self: &mut Self) -> &mut TGets a mutable reference to the underlying
Buf.It is inadvisable to directly read from the underlying
Buf.Examples
use ; let buf = from; let mut iter = buf.into_iter; assert_eq!; iter.get_mut.advance; assert_eq!;
impl<I> IntoIterator for IntoIter<T>
fn into_iter(self: Self) -> I
impl<T> Any for IntoIter<T>
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for IntoIter<T>
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for IntoIter<T>
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> Freeze for IntoIter<T>
impl<T> From for IntoIter<T>
fn from(t: T) -> TReturns the argument unchanged.
impl<T> RefUnwindSafe for IntoIter<T>
impl<T> Send for IntoIter<T>
impl<T> Sync for IntoIter<T>
impl<T> Unpin for IntoIter<T>
impl<T> UnsafeUnpin for IntoIter<T>
impl<T> UnwindSafe for IntoIter<T>
impl<T, U> Into for IntoIter<T>
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for IntoIter<T>
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for IntoIter<T>
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>
impl<T: $crate::fmt::Debug> Debug for IntoIter<T>
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl<T: Buf> ExactSizeIterator for IntoIter<T>
impl<T: Buf> Iterator for IntoIter<T>
fn next(self: &mut Self) -> Option<u8>fn size_hint(self: &Self) -> (usize, Option<usize>)